@@ -37,6 +37,10 @@
 # 2015-04-30: don't allow error.estimate="dispersion" for F-tests in binomial
 #             and Poission GLMs. John
 # 2015-08-29: fixed Anova() for coxph models with clusters. John
+# 2015-09-04: added support for coxme models. John
+# 2015-09-11: modified Anova.default() to work with vglm objects from VGAM. John
+# 2015-09-15: fixed Anova.default() so that F-tests work again. John
+# 2015-11-13: modify Anova.coxph() to take account of method/ties argument. John
 #-------------------------------------------------------------------------------
 
 # Type II and III tests for linear, generalized linear, and other models (J. Fox)
@@ -137,7 +141,7 @@ Anova.II.lm <- function(mod, error, singular.ok=TRUE, ...){
 	not.aliased <- !is.na(coef(mod))
 	if (!singular.ok && !all(not.aliased))
 		stop("there are aliased coefficients in the model")
-	fac <- attr(mod$terms, "factors")
+	fac <- attr(terms(mod), "factors")
 	intercept <- has.intercept(mod)
 	I.p <- diag(length(coefficients(mod)))
 	assign <- mod$assign
@@ -338,7 +342,7 @@ Anova.II.LR.glm <- function(mod, singular.ok=TRUE, ...){
 		stop("there are aliased coefficients in the model")
 	# (some code adapted from drop1.glm)
 	which.nms <- function(name) which(asgn == which(names == name))
-	fac <- attr(mod$terms, "factors")
+	fac <- attr(terms(mod), "factors")
 	names <- if (has.intercept(mod)) term.names(mod)[-1]
 			else term.names(mod)
 	n.terms <- length(names)
@@ -398,7 +402,7 @@ Anova.II.F.glm <- function(mod, error, error.estimate, singular.ok=TRUE, ...){
         pearson = sum(residuals(error, "pearson")^2, na.rm=TRUE),
         dispersion = df.res*summary(error, corr = FALSE)$dispersion,
         deviance = deviance(error))
-    fac <- attr(mod$terms, "factors")
+    fac <- attr(terms(mod), "factors")
     names <- if (has.intercept(mod)) term.names(mod)[-1]
     else term.names(mod)
     n.terms <- length(names)
@@ -471,7 +475,7 @@ Anova.II.multinom <- function (mod, ...)
 {
 	which.nms <- function(name) which(asgn == which(names ==
 								name))
-	fac <- attr(mod$terms, "factors")
+	fac <- attr(terms(mod), "factors")
 	names <- if (has.intercept(mod)) term.names(mod)[-1]
 			else term.names(mod)
 	n.terms <- length(names)
@@ -558,7 +562,7 @@ Anova.II.polr <- function (mod, ...)
   if (!requireNamespace("MASS")) stop("MASS package is missing")
 	which.nms <- function(name) which(asgn == which(names ==
 								name))
-	fac <- attr(mod$terms, "factors")
+	fac <- attr(terms(mod), "factors")
 	names <- term.names(mod)
 	n.terms <- length(names)
 	X <- model.matrix(mod)
@@ -804,7 +808,7 @@ Anova.II.mlm <- function(mod, SSPE, error.df, idata, idesign, icontrasts, imatri
 			return(list(SSP = lh2$SSPH - SSP1, SSPE=lh2$SSPE, P=lh2$P, singular=lh2$singular))
 		}
 	}
-	fac <- attr(mod$terms, "factors")
+	fac <- attr(terms(mod), "factors")
 	intercept <- has.intercept(mod)
 	p <- nrow(coefficients(mod))
 	I.p <- diag(p)
@@ -1129,6 +1133,7 @@ Anova.II.LR.coxph <- function(mod, ...){
 	names <- term.names(mod)
 	n.terms <- length(names)
 	if (n.terms < 2) return(anova(mod, test="Chisq"))
+	method <- mod$method
 	X <- model.matrix(mod)
 	asgn <- attr(X, 'assign')
 	p <- LR <- rep(0, n.terms)
@@ -1136,12 +1141,12 @@ Anova.II.LR.coxph <- function(mod, ...){
 	for (term in 1:n.terms){
 		rels <- names[relatives(names[term], names, fac)]
 		exclude.1 <- as.vector(unlist(sapply(c(names[term], rels), which.nms)))
-		mod.1 <- survival::coxph(mod$y ~ X[, -exclude.1, drop = FALSE])
+		mod.1 <- survival::coxph(mod$y ~ X[, -exclude.1, drop = FALSE], method=method)
 		loglik.1 <- logLik(mod.1)
 		mod.2 <- if (length(rels) == 0) mod
 				else {
 					exclude.2 <- as.vector(unlist(sapply(rels, which.nms)))
-					survival::coxph(mod$y ~ X[, -exclude.2, drop = FALSE])
+					survival::coxph(mod$y ~ X[, -exclude.2, drop = FALSE], method=method)
 				}
 		loglik.2 <- logLik(mod.2)
 		LR[term] <- -2*(loglik.1 - loglik.2)
@@ -1162,13 +1167,14 @@ Anova.III.LR.coxph <- function(mod, ...){
 	names <- term.names(mod)
 	n.terms <- length(names)
 	if (n.terms < 2) return(anova(mod, test="Chisq"))
+	method <- mod$method
 	X <- model.matrix(mod)
 	asgn <- attr(X, 'assign')
 	df <- df.terms(mod)
 	LR <- p <- rep(0, n.terms)
 	loglik1 <- logLik(mod)
 	for (term in 1:n.terms){
-		mod.0 <- survival::coxph(mod$y ~ X[, -which.nms(names[term])])
+		mod.0 <- survival::coxph(mod$y ~ X[, -which.nms(names[term])], method=method)
 		LR[term] <- -2*(logLik(mod.0) - loglik1)
 		p[term] <- pchisq(LR[term], df[term], lower.tail=FALSE)
 	}
@@ -1333,14 +1339,26 @@ Anova.default <- function(mod, type=c("II","III", 2, 3), test.statistic=c("Chisq
 			"3"=Anova.III.default(mod, vcov., test.statistic, singular.ok=singular.ok))
 }
 
+assignVector <- function(model){
+    m <- model.matrix(model)
+    assign <- attr(m, "assign")
+    if (!is.null(assign)) return (assign)
+    m <- model.matrix(formula(model), data=model.frame(model))
+    assign <- attr(m, "assign")
+    if (!has.intercept(model)) assign <- assign[assign != 0]
+    assign
+}
+
 Anova.II.default <- function(mod, vcov., test, singular.ok=TRUE, ...){
 	hyp.term <- function(term){
 		which.term <- which(term==names)
-		subs.term <- which(assign==which.term)
+		subs.term <- if (is.list(assign)) assign[[which.term]] else which(assign == which.term)
 		relatives <- relatives(term, names, fac)
 		subs.relatives <- NULL
-		for (relative in relatives) 
-			subs.relatives <- c(subs.relatives, which(assign==relative))
+		for (relative in relatives){
+		    sr <- if (is.list(assign)) assign[[relative]] else which(assign == relative)
+			subs.relatives <- c(subs.relatives, sr)
+		}
 		hyp.matrix.1 <- I.p[subs.relatives,,drop=FALSE]
 		hyp.matrix.1 <- hyp.matrix.1[, not.aliased, drop=FALSE]
 		hyp.matrix.2 <- I.p[c(subs.relatives,subs.term),,drop=FALSE]
@@ -1359,12 +1377,13 @@ Anova.II.default <- function(mod, vcov., test, singular.ok=TRUE, ...){
 	not.aliased <- !is.na(coef(mod))
 	if (!singular.ok && !all(not.aliased))
 		stop("there are aliased coefficients in the model")
-	fac <- attr(mod$terms, "factors")
+	fac <- attr(terms(mod), "factors")
 	intercept <- has.intercept(mod)
 	p <- length(coefficients(mod))
 	I.p <- diag(p)
-	assign <- attr(model.matrix(mod), "assign")
-	assign[!not.aliased] <- NA
+	assign <- assignVector(mod) # attr(model.matrix(mod), "assign")
+	if (!is.list(assign)) assign[!not.aliased] <- NA
+	else if (intercept) assign <- assign[-1]
 	names <- term.names(mod)
 	if (intercept) names <- names[-1]
 	n.terms <- length(names)
@@ -1378,6 +1397,7 @@ Anova.II.default <- function(mod, vcov., test, singular.ok=TRUE, ...){
 			n.terms <- n.terms - length(clusters)
 		}
 	}
+#	if (inherits(mod, "plm")) assign <- assign[assign != 0]
 	p <- teststat <- rep(0, n.terms + 1)
 	teststat[n.terms + 1] <- p[n.terms + 1] <- NA
 	for (i in 1:n.terms){
@@ -1387,9 +1407,14 @@ Anova.II.default <- function(mod, vcov., test, singular.ok=TRUE, ...){
 		p[i] <- if (test == "Chisq") 
 					pchisq(teststat[i], df[i], lower.tail=FALSE) 
 				else pf(teststat[i], df[i], df[n.terms + 1], lower.tail=FALSE)
-	}    
-	result <- data.frame(df, teststat, p)
-	row.names(result) <- c(names,"Residuals")
+	}
+	result <- if (test == "Chisq"){ 
+	    if (length(df) == n.terms + 1) df <- df[1:n.terms]
+	    data.frame(df, teststat[!is.na(teststat)], p[!is.na(teststat)])
+	}
+	else data.frame(df, teststat, p)
+	if (nrow(result) == length(names) + 1) names <- c(names,"Residuals")
+	row.names(result) <- names
 	names(result) <- c ("Df", test, if (test == "Chisq") "Pr(>Chisq)" 
 					else "Pr(>F)")
 	class(result) <- c("anova", "data.frame")
@@ -1404,7 +1429,7 @@ Anova.III.default <- function(mod, vcov., test, singular.ok=FALSE, ...){
 	I.p <- diag(p)
 	names <- term.names(mod)
 	n.terms <- length(names)
-	assign <- attr(model.matrix(mod), "assign")
+	assign <- assignVector(mod) # attr(model.matrix(mod), "assign")
 	df <- c(rep(0, n.terms), df.residual(mod))
 	if (inherits(mod, "coxph")){
 		if (intercept) names <- names[-1]
@@ -1416,7 +1441,8 @@ Anova.III.default <- function(mod, vcov., test, singular.ok=FALSE, ...){
 			n.terms <- n.terms - length(clusters)
 		}
 	}
-	if (intercept) df[1] <- 1
+#	if (inherits(mod, "plm")) assign <- assign[assign != 0]
+	if (intercept) df[1] <- sum(grepl("^\\(Intercept\\)", names(coef(mod))))
 	teststat <- rep(0, n.terms + 1)
 	p <- rep(0, n.terms + 1)
 	teststat[n.terms + 1] <- p[n.terms + 1] <- NA
@@ -1424,7 +1450,7 @@ Anova.III.default <- function(mod, vcov., test, singular.ok=FALSE, ...){
 	if (!singular.ok && !all(not.aliased))
 		stop("there are aliased coefficients in the model")
 	for (term in 1:n.terms){
-		subs <- which(assign == term - intercept)        
+		subs <- if (is.list(assign)) assign[[term]] else which(assign == term - intercept)    
 		hyp.matrix <- I.p[subs,,drop=FALSE]
 		hyp.matrix <- hyp.matrix[, not.aliased, drop=FALSE]
 		hyp.matrix <- hyp.matrix[!apply(hyp.matrix, 1, function(x) all(x == 0)), , drop=FALSE]        
@@ -1443,8 +1469,13 @@ Anova.III.default <- function(mod, vcov., test, singular.ok=FALSE, ...){
 					else pf(teststat[term], df[term], df[n.terms + 1], lower.tail=FALSE)
 		}
 	}
-	result <- data.frame(df, teststat, p)
-	row.names(result) <- c(names, "Residuals")
+	result <- if (test == "Chisq"){ 
+	    if (length(df) == n.terms + 1) df <- df[1:n.terms]
+	    data.frame(df, teststat[!is.na(teststat)], p[!is.na(teststat)])
+	}
+	else data.frame(df, teststat, p)
+	if (nrow(result) == length(names) + 1) names <- c(names,"Residuals")
+	row.names(result) <- names
 	names(result) <- c ("Df", test, if (test == "Chisq") "Pr(>Chisq)" 
 					else "Pr(>F)")
 	class(result) <- c("anova", "data.frame")
@@ -1735,3 +1766,60 @@ Anova.III.lme <- function(mod, vcov., singular.ok=FALSE, ...){
 Anova.svyglm <- function(mod, ...) Anova.default(mod, ...)
 
 Anova.rlm <- function(mod, ...) Anova.default(mod, test.statistic="F", ...)
+
+Anova.coxme <- function(mod, type=c("II","III", 2, 3), test.statistic=c("Wald", "LR"), ...){
+    type <- as.character(type)
+    type <- match.arg(type)
+    test.statistic <- match.arg(test.statistic)
+    switch(type,
+        II=switch(test.statistic,
+            LR=Anova.II.LR.coxme(mod, ...),
+            Wald=Anova.default(mod, type="II", test.statistic="Chisq", ...)),
+        III=switch(test.statistic,
+            LR=stop("type-III LR tests not available for coxme models"),
+            Wald=Anova.default(mod, type="III", test.statistic="Chisq", ...)),
+        "2"=switch(test.statistic,
+            LR=Anova.II.LR.coxme(mod, ...),
+            Wald=Anova.default(mod, type="II", test.statistic="Chisq", ...)),
+        "3"=switch(test.statistic,
+            LR=stop("type-III LR tests not available for coxme models"),
+            Wald=Anova.default(mod, type="III", test.statistic="Chisq")))
+}
+
+Anova.II.LR.coxme <- function(mod, ...){
+    if (!requireNamespace("coxme")) stop("coxme package is missing")
+    which.nms <- function(name) which(asgn == which(names == name))
+    fac <-attr(terms(mod), "factors")
+    names <- term.names(mod)
+    n.terms <- length(names)
+    if (n.terms < 2) return(anova(mod, test="Chisq"))
+    X <- model.matrix(mod)
+    asgn <- attr(X, 'assign')
+    p <- LR <- rep(0, n.terms)
+    df <- df.terms(mod)
+    random <- mod$formulaList$random
+    random <- sapply(random, as.character)[2, ]
+    random <- paste(paste0("(", random, ")"), collapse=" + ")
+    fixed <- as.character(mod$formulaList$fixed)[3]
+    for (term in 1:n.terms){
+        rels <- names[relatives(names[term], names, fac)]
+        formula <- paste0(". ~ . - ", paste(c(names[term], rels), collapse=" - "), " + ", random)
+        mod.1 <- update(mod, as.formula(formula))
+        loglik.1 <- logLik(mod.1, type="integrated")
+        mod.2 <- if (length(rels) == 0) mod
+        else {
+            formula <- paste0(". ~ . - ", paste(rels, collapse=" - "), " + ", random)
+            update(mod, as.formula(formula))
+        }
+        loglik.2 <- logLik(mod.2, type="integrated")
+        LR[term] <- -2*(loglik.1 - loglik.2)
+        p[term] <- pchisq(LR[term], df[term], lower.tail=FALSE)
+    }
+    result <- data.frame(LR, df, p)
+    row.names(result) <- names
+    names(result) <- c("LR Chisq", "Df", "Pr(>Chisq)")
+    class(result) <- c("anova", "data.frame")
+    attr(result, "heading") <- "Analysis of Deviance Table (Type II tests)"
+    result
+}
+
